class Session < ApplicationRecord
  belongs_to :conference
  belongs_to :stage
  has_many :assignments
  has_many :users, through: :assignments
  has_many :candidates
  has_many :session_speakers, dependent: :destroy
  has_many :speakers, through: :session_speakers
  has_many :filedrop_comments, dependent: :destroy
  has_many :filedrop_files, dependent: :destroy

  scope :scheduled, -> { where(status: "scheduled") }
  scope :future, -> { where(starts_at: Time.now..) }

  validates :ref_id, uniqueness: { scope: :conference_id }
  # ref_id is used as part of a path, so make sure its safe
  validates :ref_id, format: { with: /\A[0-9a-fA-F-]+\z/, message: "only allows hexadecimal characters and minus" }

  after_update :notify_if_changed

  self.implicit_order_column = :starts_at

  def to_param
    ref_id
  end

  def starts_at
    super.in_time_zone(conference.time_zone)
  end

  def ends_at
    super.in_time_zone(conference.time_zone)
  end

  def is_interpreted?
    is_relevant?
  end

  def is_relevant?
    RelevantStage.exists?(stage:)
  end

  def translators_needed?
    is_interpreted? && assignments.length < 2
  end

  def backup_needed?
    false
  end

  def assignees?
    assignments.length.positive?
  end

  def filedrop?
    filedrop_files.exists? || filedrop_comments.exists?
  end

  def duration_minutes
    (ends_at - starts_at) / 60.0
  end

  private

  def notify_if_changed
    return if new_record?

    return unless saved_changes.present?

    ActiveSupport::Notifications.instrument("session.updated", record: self, changes: saved_changes)
  end
end
